==> guikachu-0.11.tar.gz: guikachu-0.11/src/form-editor/button.cc <==
//$Id: button.cc,v 1.22 2001/10/07 13:06:19 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "button.h"
#include "button-canvas.h"

#include "form-editor.h"

#include <gnome-xml/xmlmemory.h>

using Guikachu::Widgets::Button;
using namespace Guikachu::GUI::FormEditor;

Button::Button (Resources::Form *owner,
		const string& id):
    Widget (owner, id),
    Resizeable (changed),
    Textual (changed, id),
    editor (0),
    frame (changed, BUTTON_FRAME_SIMPLE),
    usable (changed, true),
    repeat (changed, false),
    anchor_right (changed, false)
{
}

Button::~Button ()
{
    if (editor)
	delete editor;
}

Gtk::Widget *Button::create_editor ()
{
    if (!editor)
	editor = new ButtonProperties (this);

    return editor->get_editor ();
}

CanvasItem* Button::create_canvas_item (GUI::FormEditor::Form *form)
{
    return new ButtonCanvasItem (this, form);
}


bool Button::load (StorageNode &node)
{
    /* Non-NULL default values */
    usable = false;
    frame = BUTTON_FRAME_NONE;
    
    for (StorageNode curr_node  = node.children ();
	 curr_node; curr_node++)
    {
	if (curr_node.name () == "pos")
	{
	    x = curr_node.get_prop<int> ("x");
	    y = curr_node.get_prop<int> ("y");

	    string width_str = curr_node.get_prop<string> ("width");
	    if (width_str == "auto")
	    {
		manual_width = false;
		width = 0;
	    } else {
		manual_width = true;
		width = atoi (width_str.c_str ());
	    }

	    string height_str = curr_node.get_prop<string> ("height");
	    if (height_str == "auto")
	    {
		manual_height = false;
		height = 0;
	    } else {
		manual_height = true;
		height = atoi (height_str.c_str ());
	    }
	}
	else if (curr_node.name () == "text")
	{
	    text = curr_node.get_content ();
	    font = curr_node.get_prop <int> ("font");
	}
	else if (curr_node.name () == "frame")
	{
	    if (curr_node.get_prop<string> ("bold") != "")
		frame = BUTTON_FRAME_BOLD;
	    else
		frame = BUTTON_FRAME_SIMPLE;
	}
	else if (curr_node.name () == "usable")
	    usable = true;
	else if (curr_node.name () == "repeat")
	    repeat = true;
	else if (curr_node.name () == "anchor_right")
	    anchor_right = true;
	else if (curr_node.name () == "usable")
	    usable = true;
	else
	    g_warning ("Error parsing `button' widget: "
		       "unexpected element `%s'",
		       curr_node.name ().c_str ());
    }

    changed ();

    return true;
}

void Button::save (StorageNode &node) const
{
    StorageNode my_node;
    
    /* Position */
    my_node = node.add_node ("pos");
    my_node.set_prop ("x", x);
    my_node.set_prop ("y", y);

    /* Size */
    if (manual_width)
	my_node.set_prop ("width", width);
    else
	my_node.set_prop ("width", "auto");

    if (manual_height)
	my_node.set_prop ("height", height);
    else
	my_node.set_prop ("height", "auto");
    
    /* Frame */
    if (frame != BUTTON_FRAME_NONE)
    {
	my_node = node.add_node ("frame");
	if (frame == BUTTON_FRAME_BOLD)
	    my_node.set_prop ("bold", "bold");
    }
    
    /* Usable */
    if (usable)
	node.add_node ("usable");

    /* Repeating */
    if (repeat)
	node.add_node ("repeat");

    /* Anchor_right */
    if (anchor_right)
	node.add_node ("anchor_right");
    
    /* Label and font */
    my_node = node.add_node ("text", text);
    my_node.set_prop ("font", font);
}

void Button::save_rcp (ostream &ostr) const
{
    string frame_str, anchor_str;
    gchar *width_str, *height_str;
    string type_str;

    if (!repeat)
	type_str = "BUTTON";
    else
	type_str = "REPEATBUTTON";
    
    if (manual_width)
	width_str = g_strdup_printf ("%d", int (width));
    else
	width_str = g_strdup ("AUTO");
    
    if (manual_height)
	height_str = g_strdup_printf ("%d", int (height));
    else
	height_str = g_strdup ("AUTO");
    
    switch (frame)
    {
    case BUTTON_FRAME_NONE:
	frame_str = "NOFRAME";
	break;
    case BUTTON_FRAME_SIMPLE:
	frame_str = "FRAME";
	break;
    case BUTTON_FRAME_BOLD:
	frame_str = "BOLDFRAME";
	break;
    }

    if (anchor_right)
	anchor_str = "RIGHTANCHOR";
    else
	anchor_str = "LEFTANCHOR";
    
    ostr << "  " << type_str << " \"" << text << "\" ID " << id;
    ostr << " AT (" << x << " " << y << " " << width_str << " " << height_str << ")";
    ostr << " FONT " << font;
    ostr << " " << frame_str;
    ostr << " " << anchor_str;
    if (usable)
	ostr << " USABLE";
    else
	ostr << " NONUSABLE";
    
    ostr << endl;

    g_free (height_str);
    g_free (width_str);
}

int Button::get_display_width ()
{
    if (manual_width)
	return width;

    int text_width = get_string_width (font, text);
    return 2 + text_width + 2;
}

int Button::get_display_height ()
{
    if (manual_height)
	return height;

    int text_height = get_font_height (font);
    return text_height + 2;
}

==> guikachu-0.12.tar.gz: guikachu-0.12/src/form-editor/button.cc <==
//$Id: button.cc,v 1.23 2001/10/23 13:05:51 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "button.h"
#include "button-canvas.h"

#include "form-editor.h"

#include <gnome-xml/xmlmemory.h>

using Guikachu::Widgets::Button;
using namespace Guikachu::GUI::FormEditor;

Button::Button (Resources::Form *owner,
		const string& id):
    Widget (owner, id),
    Resizeable (changed),
    Textual (changed, id),
    editor (0),
    frame (changed, BUTTON_FRAME_SIMPLE),
    usable (changed, true),
    repeat (changed, false),
    anchor_right (changed, false)
{
}

Button::~Button ()
{
    if (editor)
	delete editor;
}

Gtk::Widget *Button::create_editor ()
{
    if (!editor)
	editor = new ButtonProperties (this);

    return editor->get_editor ();
}

CanvasItem* Button::create_canvas_item (GUI::FormEditor::Form *form)
{
    return new ButtonCanvasItem (this, form);
}


bool Button::load (StorageNode &node)
{
    /* Non-NULL default values */
    usable = false;
    frame = BUTTON_FRAME_NONE;
    
    for (StorageNode curr_node  = node.children ();
	 curr_node; curr_node++)
    {
	if (curr_node.name () == "pos")
	{
	    x = curr_node.get_prop<int> ("x");
	    y = curr_node.get_prop<int> ("y");

	    string width_str = curr_node.get_prop<string> ("width");
	    if (width_str == "auto")
	    {
		manual_width = false;
		width = 0;
	    } else {
		manual_width = true;
		width = atoi (width_str.c_str ());
	    }

	    string height_str = curr_node.get_prop<string> ("height");
	    if (height_str == "auto")
	    {
		manual_height = false;
		height = 0;
	    } else {
		manual_height = true;
		height = atoi (height_str.c_str ());
	    }
	}
	else if (curr_node.name () == "text")
	{
	    text = curr_node.get_content ();
	    font = curr_node.get_prop <int> ("font");
	}
	else if (curr_node.name () == "frame")
	{
	    if (curr_node.get_prop<string> ("bold") != "")
		frame = BUTTON_FRAME_BOLD;
	    else
		frame = BUTTON_FRAME_SIMPLE;
	}
	else if (curr_node.name () == "usable")
	    usable = true;
	else if (curr_node.name () == "repeat")
	    repeat = true;
	else if (curr_node.name () == "anchor_right")
	    anchor_right = true;
	else if (curr_node.name () == "usable")
	    usable = true;
	else
	    g_warning ("Error parsing `button' widget: "
		       "unexpected element `%s'",
		       curr_node.name ().c_str ());
    }

    changed ();

    return true;
}

void Button::save (StorageNode &node) const
{
    StorageNode my_node;
    
    /* Position */
    my_node = node.add_node ("pos");
    my_node.set_prop ("x", x);
    my_node.set_prop ("y", y);

    /* Size */
    if (manual_width)
	my_node.set_prop ("width", width);
    else
	my_node.set_prop ("width", "auto");

    if (manual_height)
	my_node.set_prop ("height", height);
    else
	my_node.set_prop ("height", "auto");
    
    /* Frame */
    if (frame != BUTTON_FRAME_NONE)
    {
	my_node = node.add_node ("frame");
	if (frame == BUTTON_FRAME_BOLD)
	    my_node.set_prop ("bold", "bold");
    }
    
    /* Usable */
    if (usable)
	node.add_node ("usable");

    /* Repeating */
    if (repeat)
	node.add_node ("repeat");

    /* Anchor_right */
    if (anchor_right)
	node.add_node ("anchor_right");
    
    /* Label and font */
    my_node = node.add_node ("text", text);
    my_node.set_prop ("font", font);
}

void Button::save_rcp (ostream &ostr) const
{
    string frame_str, anchor_str;
    gchar *width_str, *height_str;
    string type_str;

    if (!repeat)
	type_str = "BUTTON";
    else
	type_str = "REPEATBUTTON";
    
    if (manual_width)
	width_str = g_strdup_printf ("%d", int (width));
    else
	width_str = g_strdup ("AUTO");
    
    if (manual_height)
	height_str = g_strdup_printf ("%d", int (height));
    else
	height_str = g_strdup ("AUTO");
    
    switch (frame)
    {
    case BUTTON_FRAME_NONE:
	frame_str = "NOFRAME";
	break;
    case BUTTON_FRAME_SIMPLE:
	frame_str = "FRAME";
	break;
    case BUTTON_FRAME_BOLD:
	frame_str = "BOLDFRAME";
	break;
    }

    if (anchor_right)
	anchor_str = "RIGHTANCHOR";
    else
	anchor_str = "LEFTANCHOR";
    
    ostr << "  " << type_str << " \"" << text << "\" ID " << id;
    ostr << " AT (" << x << " " << y << " " << width_str << " " << height_str << ")";
    ostr << " FONT " << font;
    ostr << " " << frame_str;
    ostr << " " << anchor_str;
    if (usable)
	ostr << " USABLE";
    else
	ostr << " NONUSABLE";
    
    ostr << endl;

    g_free (height_str);
    g_free (width_str);
}

int Button::get_auto_width ()
{
    int text_width = get_string_width (font, text);
    return 2 + text_width + 2;
}

int Button::get_auto_height ()
{
    int text_height = get_font_height (font);
    return text_height + 2;
}
